// Code for clock on an oscilloscope in X-Y mode
//
// By howardlong (eevblog id) free to use for any purpose, but please note any other third party licensing conditions
// in particular in the fonts.h file
//
// Designed for PIC24FV16KM202 on 5V Microstick board DM240013-2
//
// Pin 23 is the X axis to oscilloscope
// Pin 25 is the Y axis to oscilloscope
// Pins 8 or 27 can be used for ground leads, or the black probe point on the board
//
// Three buttons to ground can set the date and time (or you can set the INIT_YEAR... etc and recompile)
//		RB5, pin 14, selects Year, Month, etc. 
//		RB6, pin 15, decrements the selected value
//		RB7, pin 16, increments the selected value
//
// Accuracy can be improved with a 32768Hz xtal in position Y2 and two 22pF 0603 caps in C19 and C21,
// and changing to use the secondary oscillator instead of the low power RC internal oscillator, 
// see RTCC_CLOCK_SOURCE macro below and comment out as appropriate.

#define RTCC_CLOCK_SOURCE 0b01; // Internal LPRC, use if you don't have a xtal Y2 and caps C19 & C21
//#define RTCC_CLOCK_SOURCE 0b00; // SOSC, use this if you have Y2 xtal and caps, much more accurate time keeping

#define STATIC_TEXT "EEVBlog" // This appears at the first line
#define STATIC_TEXT_XPOS 68 // Whereabouts on the line the static text starts drawing (0-255)

#define INIT_YEAR 14 // Comment this line out to prevent resetting of RTCC each time unit is started, note YY is the format
#define INIT_MONTH 12 // If you can't be bothered to add three buttons then this is the way to set the clock
#define INIT_DAY 21
#define INIT_HOUR 14
#define INIT_MINUTE 16
#define INIT_SECOND 0

#include <xc.h>
#include <stdint.h>
#include <ctype.h>
#include <string.h>
#include <math.h>

#define _MAIN
#include "main.h"
#include "font.h"

#pragma config FNOSC = FRC
#pragma config SOSCSRC = ANA
#pragma config POSCMOD = NONE
#pragma config OSCIOFNC = CLKO
#pragma config SOSCSEL = SOSCHP
#pragma config FCKSM = CSECMD
#pragma config FWDTEN = OFF
#pragma config MCLRE = ON
#pragma config ICS = PGx1

#define abs(x) ((x)<0?-(x):(x))
#define BCDTOBINARY(x) ((((x) >> 4) * 10) + ((x) & 0x0F))
#define BINARYTOBCD(x) ((((x) / 10) << 4) + ((x) % 10 ))
#define PI 3.14159265d

#define BTNNUM 6
#define BTNMASK0 (1<<5)
#define BTNMASK1 (1<<6)
#define BTNMASK2 (1<<8)
#define BTNMASK3 (1<<9)
#define BTNMASK4 (1<<10)
#define BTNMASK5 (1<<11)
#define BTNMASKALL (BTNMASK0 | BTNMASK1 | BTNMASK2 | BTNMASK3 | BTNMASK4 | BTNMASK5)

#define LINE1Y 80
#define LINE2Y 114
#define LINE3Y 148
#define UNDERLINEYOFFSET 29

typedef enum
{
	DTSE_YEAR,
	DTSE_MONTH,
	DTSE_DAY,
	DTSE_HOUR,
	DTSE_MINUTE,
	DTSE_SECOND,
	DTSE_NONE // No marker displayed
} DATETIMESELECTENUM;

typedef struct
{
	U8 u8x;
	U8 u8y;
} VERTEXSTRUCT;

typedef struct
{
	VERTEXSTRUCT vs1;
	VERTEXSTRUCT vs2;
} LINESTRUCT;

typedef struct
{
	U8 u8Year;
	U8 u8Month;
	U8 u8Day;
	U8 u8Hour;
	U8 u8Minute;
	U8 u8Second;
} DATETIMESTRUCT;	

static LINESTRUCT _alsClockFace[12];
static LINESTRUCT _alsClockHandSecond[60];
static LINESTRUCT _alsClockHandMinute[60];
static LINESTRUCT _alsClockHandHour[60];

static void OscInit(void)
{
	// Set FOSC to 32MHz (i.e. FCY = 16MHz)
	CLKDIVbits.RCDIV=0b000; // div by 1 FRC postscaler
	
	__builtin_write_OSCCONH(0b00000001); // NOSC=FRCPLL
	__builtin_write_OSCCONL(0x00000011); // OSWEN=1, SOSCEN=1
	
	while(OSCCONbits.COSC!=0b001)
	{
		Nop();
	}	
	while(OSCCONbits.LOCK!=1)
	{
		Nop();
	}
}

static void DACInit(void)
{
	DAC1CONbits.DACFM=0; // Lower 8 bits of DACxDAT used
	DAC1CONbits.DACTRIG=0; // DAC output updated as soon as written to
	DAC1CONbits.DACREF=0b10; // AVDD is reference
	DAC1CONbits.DACEN=1; // Enable DAC
	DAC1CONbits.DACOE=1; // DAC output enable
	
	DAC2CONbits.DACFM=0; // Lower 8 bits of DACxDAT used
	DAC2CONbits.DACTRIG=0; // DAC output updated as soon as written to
	DAC2CONbits.DACREF=0b10; // AVDD is reference
	DAC2CONbits.DACEN=1; // Enable DAC
	DAC2CONbits.DACOE=1; // DAC output enable
}	
	
static void RTCCInit(void)
{
	__builtin_write_RTCWEN(); // Special write to enable RTCC writes
	RTCPWCbits.RTCCLK=RTCC_CLOCK_SOURCE;
	RCFGCALbits.RTCEN=1;
	
	RCFGCALbits.RTCWREN=0; // Stop any more writes to RTCC
}

static void RTCCSet(DATETIMESTRUCT *pdts)
{
	// Date and time parameters are in BCD
	__builtin_write_RTCWEN(); // Special write to enable RTCC writes
	
	RCFGCALbits.RTCPTR=0b11;
	
	RTCVAL=pdts->u8Year;
	RTCVAL=(((U16)(pdts->u8Month))<<8) + pdts->u8Day;
	RTCVAL=pdts->u8Hour;
	RTCVAL=(((U16)(pdts->u8Minute))<<8) + pdts->u8Second;
	
	RCFGCALbits.RTCWREN=0; // Stop any more writes to RTCC
}

static void RTCCGet(DATETIMESTRUCT *pdts)
{
	// Returns BCD values in *pdts structure
	U16 u16;
	
	RCFGCALbits.RTCPTR=0b11; // Note this auto decrements for each RTCVALH read
	
	u16=RTCVAL;
	pdts->u8Year=(U8)u16;
	u16=RTCVAL;
	pdts->u8Month=(U8)(u16>>8);
	pdts->u8Day=(U8)u16;
	u16=RTCVAL;
	pdts->u8Hour=(U8)(u16);
	u16=RTCVAL;
	pdts->u8Minute=(U8)(u16>>8);
	pdts->u8Second=(U8)(u16);
}	

static void RTCCBCDToBinary(DATETIMESTRUCT *pdts)
{
	// Put numbers in binary format rather than the RTCC's BCD format
	pdts->u8Second=BCDTOBINARY(pdts->u8Second);
	pdts->u8Minute=BCDTOBINARY(pdts->u8Minute);
	pdts->u8Hour=BCDTOBINARY(pdts->u8Hour);
	pdts->u8Day=BCDTOBINARY(pdts->u8Day);
	pdts->u8Month=BCDTOBINARY(pdts->u8Month);
	pdts->u8Year=BCDTOBINARY(pdts->u8Year);
}	

static void RTCCBinaryToBCD(DATETIMESTRUCT *pdts)
{
	// Convert binary values back to RTCC BCD format
	pdts->u8Year=BINARYTOBCD(pdts->u8Year);
	pdts->u8Month=BINARYTOBCD(pdts->u8Month);
	pdts->u8Day=BINARYTOBCD(pdts->u8Day);
	pdts->u8Hour=BINARYTOBCD(pdts->u8Hour);
	pdts->u8Minute=BINARYTOBCD(pdts->u8Minute);
	pdts->u8Second=BINARYTOBCD(pdts->u8Second);
}	

static void RTCCInc(DATETIMESELECTENUM dtse)
{
	DATETIMESTRUCT dts;
	
	RTCCGet(&dts);
	
	RTCCBCDToBinary(&dts);

	switch (dtse)
	{
		case DTSE_YEAR:
			dts.u8Year++;
			if (dts.u8Year>99)
			{
				dts.u8Year=0;
			}
			break;
		case DTSE_MONTH:
			dts.u8Month++;
			if (dts.u8Month>12)
			{
				dts.u8Month=1;
			}
			break;	
		case DTSE_DAY:
			dts.u8Day++;
			if 
			(
				dts.u8Day>31 || 
				(dts.u8Day>30 && ( dts.u8Month==4 || dts.u8Month==6 || dts.u8Month==9 || dts.u8Month==11) ) ||
				(dts.u8Day>29 && dts.u8Month==2) ||
				(dts.u8Day>28 && dts.u8Month==2 && dts.u8Year%4!=0)
			)
			{
				dts.u8Day=1;
			}
			break;	
		case DTSE_HOUR:
			dts.u8Hour++;
			if (dts.u8Hour>23)
			{
				dts.u8Hour=0;
			}
			break;
		case DTSE_MINUTE:
			dts.u8Minute++;
			if (dts.u8Minute>59)
			{
				dts.u8Minute=0;
			}
			break;
		case DTSE_SECOND:
			dts.u8Second++;
			if (dts.u8Second>59)
			{
				dts.u8Second=0;
			}
			break;
		default:
			break;
	}				
	
	RTCCBinaryToBCD(&dts);
	RTCCSet(&dts);
}	

static void RTCCDec(DATETIMESELECTENUM dtse)
{
	DATETIMESTRUCT dts;
	
	RTCCGet(&dts);
	
	RTCCBCDToBinary(&dts);

	switch (dtse)
	{
		case DTSE_YEAR:
			dts.u8Year--;
			if (dts.u8Year==255)
			{
				dts.u8Year=99;
			}
			break;
		case DTSE_MONTH:
			dts.u8Month--;
			if (dts.u8Month==0)
			{
				dts.u8Month=12;
			}
			break;	
		case DTSE_DAY:
			dts.u8Day--;
			if  (dts.u8Day==0)
			{
				if (dts.u8Month==4 || dts.u8Month==6 || dts.u8Month==9 || dts.u8Month==11)
				{
					dts.u8Day=30;
				}
				else
				{
					if (dts.u8Month==2)
					{
						if (dts.u8Year%4==0)
						{
							dts.u8Day=29;
						}	
						else
						{
							dts.u8Day=28;
						}	
					}
					else
					{
						dts.u8Day=31;
					}	
				}	
			}
			break;	
		case DTSE_HOUR:
			dts.u8Hour--;
			if (dts.u8Hour==255)
			{
				dts.u8Hour=23;
			}
			break;
		case DTSE_MINUTE:
			dts.u8Minute--;
			if (dts.u8Minute==255)
			{
				dts.u8Minute=59;
			}
			break;
		case DTSE_SECOND:
			dts.u8Second--;
			if (dts.u8Second==255)
			{
				dts.u8Second=59;
			}
			break;
		default:
			break;
	}				
	
	RTCCBinaryToBCD(&dts);
	RTCCSet(&dts);
}	

static void BtnPoll(DATETIMESELECTENUM *pdtse)
{
	// Button polling
	static U16 u16BtnLast=BTNMASKALL; // Assume no buttons pressed on startup
	U16 u16Btn=PORTB & BTNMASKALL;
	
	if (u16Btn==u16BtnLast) // Make sure we have a stable state for debounce
	{
		// Now we have debounced, see if we need to do anything
		static U16 u16BtnLastState=BTNMASKALL; // Assume no buttons pressed on startup
		U16 u16BtnChanged=u16BtnLastState ^ u16Btn;
		static const U16 au16Btn[BTNNUM]={BTNMASK0,BTNMASK1,BTNMASK2,BTNMASK3,BTNMASK4,BTNMASK5};
		int i;
		static U16 u16Timer=0; // Tiemr for autorepeat
		static BOOL bReset=TRUE; // Reset the auto repeat
		
		for (i=0;i<BTNNUM;i++)
		{
			if ((u16BtnChanged & au16Btn[i])!=0)
			{
				if ((u16Btn & au16Btn[i])==0) // Has button been pressed?
				{
					switch (i)
					{
						case 0: // Select next
							(*pdtse)++;
							if (*pdtse>DTSE_NONE) 
							{
								*pdtse=DTSE_YEAR;
							}
							break;
						case 1: // Down
							RTCCDec(*pdtse);
							break;
						case 2: // Up
							RTCCInc(*pdtse);
							break;
						default:
							break;
					}
				}
				u16Timer=TMR1; // Reset button timer in the event of any button change
			}
			else
			{ // Deal with a button held down
				if ((u16Btn & au16Btn[i])==0) // Button still down
				{
					BOOL bIncDec=FALSE;
					
					if (bReset && TMR1-u16Timer>50) // Delay to first autorepeat
					{
						bReset=FALSE;
						u16Timer=TMR1;
						bIncDec=TRUE;					
					}
					else
					{
						if (!bReset && TMR1-u16Timer>10) // Delay for subsequent autorepeat
						{
							u16Timer=TMR1;
							bIncDec=TRUE;					
						}	
					}
					if (bIncDec)
					{
						switch (i)
						{
							case 1: // Down
								RTCCDec(*pdtse);
								break;
							case 2: // Up
								RTCCInc(*pdtse);
								break;
							default:
								break;
						}
					}	
				}
			}	
		}
		
		if ((u16Btn & BTNMASKALL)==BTNMASKALL)
		{
			u16Timer=TMR1;
			bReset=TRUE;
		}	
		
		u16BtnLastState=u16Btn; // Save debounced state
	}	
	u16BtnLast=u16Btn; // Save pre-debounce state
}	

// Create vertices for clock face
static void FaceInit(void)
{
	int i;
	
	for (i=0;i<12;i++)
	{
		double dAngle=2*PI*i/12;
		double dSin=sin(dAngle);
		double dCos=cos(dAngle);
		int nLength= i%3 == 0 ? 30 : 22; // Longer tick at quarter hours
		
		_alsClockFace[i].vs1.u8x=(U8)(128+127*dSin);
		_alsClockFace[i].vs1.u8y=(U8)(128-127*dCos);
		_alsClockFace[i].vs2.u8x=(U8)(128+(127-nLength)*dSin);
		_alsClockFace[i].vs2.u8y=(U8)(128-(127-nLength)*dCos);
	}	
	for (i=0;i<60;i++)
	{
		double dAngle=2*PI*i/60;
		double dSin=sin(dAngle);
		double dCos=cos(dAngle);
		
		_alsClockHandSecond[i].vs1.u8x=(U8)(128+120*dSin);
		_alsClockHandSecond[i].vs1.u8y=(U8)(128-120*dCos);
		_alsClockHandSecond[i].vs2.u8x=(U8)(128+10*dSin);
		_alsClockHandSecond[i].vs2.u8y=(U8)(128-10*dCos);

		_alsClockHandMinute[i].vs1.u8x=(U8)(128+95*dSin);
		_alsClockHandMinute[i].vs1.u8y=(U8)(128-95*dCos);
		_alsClockHandMinute[i].vs2.u8x=(U8)(128+10*dSin);
		_alsClockHandMinute[i].vs2.u8y=(U8)(128-10*dCos);

		_alsClockHandHour[i].vs1.u8x=(U8)(128+65*dSin);
		_alsClockHandHour[i].vs1.u8y=(U8)(128-65*dCos);
		_alsClockHandHour[i].vs2.u8x=(U8)(128+10*dSin);
		_alsClockHandHour[i].vs2.u8y=(U8)(128-10*dCos);
	}	
}

static void SetPixel(int nx,int ny)
{
	DAC1DAT=nx;
	DAC2DAT=255-ny;
}	

// Bressenham's pinched directly from Wikipedia.
static void DrawLine(int x0, int y0, int x1, int y1) 
{
	int dx = abs(x1-x0);
	int sx = x0<x1 ? 1 : -1;
	int dy = abs(y1-y0);
	int sy = y0<y1 ? 1 : -1; 
	int err = (dx>dy ? dx : -dy)/2;
	int e2;
	
	for(;;)
	{
		SetPixel(x0,y0);
		if (x0==x1 && y0==y1) 
		{
			break;
		}
		e2 = err;
		if (e2 >-dx)
		{ 
			err -= dy; 
			x0 += sx; 
		}
		if (e2 < dy) 
		{ 
			err += dx; 
			y0 += sy; 
		}
	}
}

static void DrawChar(int *pxPos,int *pyPos, char c)
{
	// Draw a single character at the given position, and update the position. 
	// Also deals with line wrap, and CR/LF/TAB
	if (c>=32 && c<=126)
	{
		int x1,y1,x2,y2;
		int nIdx=c-32; // Index into font array
		int nNumVerts=_au8FontNumBytes[nIdx]/4;
		const U8 *pu8=_apau8Font[nIdx];
		int i;
		int xPos=*pxPos; // Local copies to speed things up a little
		int yPos=*pyPos;
		
		if (xPos+_au8FontRealWidth[nIdx]>255) // Check to see if we neet to move to the next line
		{
			xPos=0; // Wrap horizontally
			yPos+=_u8FontHeight; // Move to next line
			if (yPos+_u8FontHeight>255)
			{
				yPos=0; // Wrap vertically
			}	
			*pyPos=yPos;
		}	
		
		for (i=0;i<nNumVerts;i++)
		{
			x1=*pu8++ + xPos;	
			y1=*pu8++ + yPos;	
			x2=*pu8++ + xPos;	
			y2=*pu8++ + yPos;
			
			DrawLine(x1,y1,x2,y2);
		}
		
		*pxPos=xPos+_au8FontWidth[nIdx]; // Move to next character position for next time
	}
	else
	{
		switch (c)
		{
			case '\r':
				*pxPos=0;
				break;
			case '\t':
				*pxPos=(*pxPos+32) & 0xE0; // tab position
				if (*pxPos<255)
				{
					break;
				}
				// Deliberately NO BREAK HERE!!!
			case '\n':
				*pyPos=*pyPos+_u8FontHeight;
				if (*pyPos+_u8FontHeight>255)
				{
					*pyPos=0; // Wrap vertically
				}
				break;
		}		
	}	
}	

static void DrawStr(int *pxPos,int *pyPos,char *psz)
{
	char c;
	
	while ((c=*psz++)!='\0')
	{
		DrawChar(pxPos,pyPos,c);
	}
}	

static void DrawStaticText(void)
{
	// First line of display, plain text
	int xPos=STATIC_TEXT_XPOS;
	int yPos=LINE1Y;
	char sz[]=STATIC_TEXT;
	
	DrawStr(&xPos,&yPos,sz);				
}	

static void DrawDateTimeText(void)
{ // Second line, YY MM DD, third line HH:MM:SS
	char sz[20];
	char *psz;
	int xPos;
	int yPos;
	DATETIMESTRUCT dts;

	RTCCGet(&dts);
	
	psz=sz;
	*psz++=(dts.u8Year>>4) + '0';
	*psz++=(dts.u8Year & 0x0F) + '0';
	*psz++=' ';
	*psz++=(dts.u8Month>>4) + '0';
	*psz++=(dts.u8Month & 0x0F) + '0';
	*psz++=' ';
	*psz++=(dts.u8Day>>4) + '0';
	*psz++=(dts.u8Day & 0x0F) + '0';
	*psz++='\0';
	xPos=53;
	yPos=LINE2Y;
	DrawStr(&xPos,&yPos,sz);

	psz=sz;
	*psz++=(dts.u8Hour>>4) + '0';
	*psz++=(dts.u8Hour & 0x0F) + '0';
	*psz++=':';
	*psz++=(dts.u8Minute>>4) + '0';
	*psz++=(dts.u8Minute & 0x0F) + '0';
	*psz++=':';
	*psz++=(dts.u8Second>>4) + '0';
	*psz++=(dts.u8Second & 0x0F) + '0';
	*psz++='\0';
	xPos=60;
	yPos=LINE3Y;
	DrawStr(&xPos,&yPos,sz);
}

static void DrawFace(void)
{ // Draw the clock face
	int i;
	LINESTRUCT *pls=_alsClockFace;
	
	for (i=0;i<12;i++) // Draw the clock face
	{
		DrawLine
		(
			pls->vs1.u8x,
			pls->vs1.u8y,
			pls->vs2.u8x,
			pls->vs2.u8y
		);
		pls++;
	}	
}

static void DrawHands(void)
{ // Draw the hands
	LINESTRUCT *pls;
	DATETIMESTRUCT dts;
	int i;
	
	RTCCGet(&dts);
	RTCCBCDToBinary(&dts);

	// Second hand
	pls=_alsClockHandSecond+dts.u8Second;
	DrawLine
	(
		pls->vs1.u8x,
		pls->vs1.u8y,
		pls->vs2.u8x,
		pls->vs2.u8y
	);
	
	// Minute hand
	pls=_alsClockHandMinute+dts.u8Minute;
	for (i=0;i<5;i++) // Make it bold by redrawing
	{
		DrawLine
		(
			pls->vs1.u8x,
			pls->vs1.u8y,
			pls->vs2.u8x,
			pls->vs2.u8y
		);
	}
	
	// Hour hand
	pls=_alsClockHandHour+(dts.u8Hour%12)*5+dts.u8Minute/12;
	for (i=0;i<5;i++) // Make it bold by redrawing
	{
		DrawLine
		(
			pls->vs1.u8x,
			pls->vs1.u8y,
			pls->vs2.u8x,
			pls->vs2.u8y
		);
	}
}	

static void DrawAdjustMarker(DATETIMESELECTENUM dtse)
{
	// Display any date/time adjustment marker required
	if (dtse!=DTSE_NONE)
	{
		const LINESTRUCT als[DTSE_NONE]=
		{
			{ {53,LINE2Y+UNDERLINEYOFFSET},{91,LINE2Y+UNDERLINEYOFFSET} },
			{ {109,LINE2Y+UNDERLINEYOFFSET},{147,LINE2Y+UNDERLINEYOFFSET} },
			{ {165,LINE2Y+UNDERLINEYOFFSET},{203,LINE2Y+UNDERLINEYOFFSET} },
			{ {60,LINE3Y+UNDERLINEYOFFSET},{98,LINE3Y+UNDERLINEYOFFSET} },
			{ {110,LINE3Y+UNDERLINEYOFFSET},{148,LINE3Y+UNDERLINEYOFFSET} },
			{ {160,LINE3Y+UNDERLINEYOFFSET},{198,LINE3Y+UNDERLINEYOFFSET} }
		};
		const LINESTRUCT *pls=als+dtse;
		
		// Draw an underline under the correct date/time part
		DrawLine
		(
			pls->vs1.u8x,
			pls->vs1.u8y,
			pls->vs2.u8x,
			pls->vs2.u8y
		);
	}	
}	

static void BtnInit(void)
{
	// Set button ports to inputs
	TRISBbits.TRISB5=1;
	TRISBbits.TRISB6=1;
	TRISBbits.TRISB8=1;
	TRISBbits.TRISB9=1;
	TRISBbits.TRISB10=1;
	TRISBbits.TRISB11=1;
	
	// Enable weak pullups
	CNPU2bits.CN27PUE=1;
	CNPU2bits.CN24PUE=1;
	CNPU2bits.CN22PUE=1;
	CNPU2bits.CN21PUE=1;
	CNPU2bits.CN16PUE=1;
	CNPU1bits.CN15PUE=1;
}	

static void TimerInit(void)
{
	// Timer used for button press timeouts
	T1CONbits.TCS=1; // TECS bits define oscillator source
	T1CONbits.TECS=0b10; // Use LPRC, so with 256 prescale tick is about 120Hz
	T1CONbits.TCKPS=0b11; // 256 prescale
	T1CONbits.TON=1; // Timer on
}	

int main(void)
{
	OscInit(); // Set 32MHz FOSC (i.e. FCY=16MHz)
	
	ANSA=0; // All I/Os digital
	ANSB=0;

	TRISBbits.TRISB15=0; // Debug output pin

	TimerInit();
	BtnInit();
	DACInit();
	RTCCInit();
	FaceInit();
	
#ifdef INIT_YEAR // Initialise the RTCC date and time if necessary
	{
		DATETIMESTRUCT dts;
		
		dts.u8Year=INIT_YEAR;
		dts.u8Month=INIT_MONTH;
		dts.u8Day=INIT_DAY;
		dts.u8Hour=INIT_HOUR;
		dts.u8Minute=INIT_MINUTE;
		dts.u8Second=INIT_SECOND;
		
		RTCCBinaryToBCD(&dts);
		RTCCSet(&dts);
	}
#endif

	while (1) // Main superloop
	{
		static DATETIMESELECTENUM dtse=DTSE_NONE; // Variable to indicate which part of date/time to update
		
		DrawStaticText();
		DrawDateTimeText();
		DrawFace();
		DrawHands();
		DrawAdjustMarker(dtse);
		BtnPoll(&dtse);
		
		LATBbits.LATB15=!LATBbits.LATB15; // Toggle this line to determine frame rate
	}		
	return 0;
}
